@@ -43,7 +43,7 @@ def _init_lazy_if_proper(results, lazy):
 
 
 @PIPELINES.register_module()
-class Fuse(object):
+class Fuse:
     """Fuse lazy operations.
 
     Fusion order:
@@ -87,7 +87,201 @@ class Fuse(object):
 
 
 @PIPELINES.register_module()
-class RandomCrop(object):
+class RandomScale:
+    """Resize images by a random scale.
+
+    Required keys are "imgs", "img_shape", "modality", added or modified
+    keys are "imgs", "img_shape", "keep_ratio", "scale_factor", "lazy",
+    "scale", "resize_size". Required keys in "lazy" is None, added or
+    modified key is "interpolation".
+
+    Args:
+        scales (tuple[int]): Tuple of scales to be chosen for resize.
+        mode (str): Selection mode for choosing the scale. Options are "range"
+            and "value". If set to "range", The short edge will be randomly
+            chosen from the range of minimum and maximum on the shorter one
+            in all tuples. Otherwise, the longer edge will be randomly chosen
+            from the range of minimum and maximum on the longer one in all
+            tuples. Default: 'range'.
+    """
+
+    def __init__(self, scales, mode='range', **kwargs):
+        self.mode = mode
+        if self.mode not in ['range', 'value']:
+            raise ValueError(f"mode should be 'range' or 'value', "
+                             f'but got {self.mode}')
+        self.scales = scales
+        self.kwargs = kwargs
+
+    def select_scale(self, scales):
+        num_scales = len(scales)
+        if num_scales == 1:
+            # specify a fixed scale
+            scale = scales[0]
+        elif num_scales == 2:
+            if self.mode == 'range':
+                scale_long = [max(s) for s in scales]
+                scale_short = [min(s) for s in scales]
+                long_edge = np.random.randint(
+                    min(scale_long),
+                    max(scale_long) + 1)
+                short_edge = np.random.randint(
+                    min(scale_short),
+                    max(scale_short) + 1)
+                scale = (long_edge, short_edge)
+            elif self.mode == 'value':
+                scale = random.choice(scales)
+        else:
+            if self.mode != 'value':
+                raise ValueError("Only 'value' mode supports more than "
+                                 '2 image scales')
+            scale = random.choice(scales)
+
+        return scale
+
+    def __call__(self, results):
+        scale = self.select_scale(self.scales)
+        results['scale'] = scale
+        resize = Resize(scale, **self.kwargs)
+        results = resize(results)
+        return results
+
+    def __repr__(self):
+        repr_str = (f'{self.__class__.__name__}('
+                    f'scales={self.scales}, mode={self.mode})')
+        return repr_str
+
+
+# Note, entity box transfroms are not added to: ThreeCrop, TenCrop,
+# MultiGroupCrop.
+@PIPELINES.register_module()
+class EntityBoxRescale:
+    """Rescale the entity box and proposals according to the image shape.
+
+    Required keys are "proposals", "gt_bboxes", added or modified keys are
+    "gt_bboxes". If original "proposals" is not None, "proposals" and
+    will be added or modified.
+
+    Args:
+        scale_factor (np.ndarray): The scale factor used entity_box rescaling.
+    """
+
+    def __init__(self, scale_factor):
+        self.scale_factor = scale_factor
+
+    def __call__(self, results):
+        scale_factor = np.concatenate([self.scale_factor, self.scale_factor])
+
+        proposals = results['proposals']
+        gt_bboxes = results['gt_bboxes']
+        results['gt_bboxes'] = gt_bboxes * scale_factor
+
+        if proposals is not None:
+            assert proposals.shape[1] == 4, (
+                'proposals shape should be in '
+                f'(n, 4), but got {proposals.shape}')
+            results['proposals'] = proposals * scale_factor
+
+        return results
+
+    def __repr__(self):
+        return f'{self.__class__.__name__}(scale_factor={self.scale_factor})'
+
+
+@PIPELINES.register_module()
+class EntityBoxCrop:
+    """Crop the entity boxes and proposals according to the cropped images.
+
+    Required keys are "proposals", "gt_bboxes", added or modified keys are
+    "gt_bboxes". If original "proposals" is not None, "proposals" will be
+    modified.
+
+    Args:
+        crop_bbox(np.ndarray | None): The bbox used to crop the original image.
+    """
+
+    def __init__(self, crop_bbox):
+        self.crop_bbox = crop_bbox
+
+    def __call__(self, results):
+        proposals = results['proposals']
+        gt_bboxes = results['gt_bboxes']
+
+        if self.crop_bbox is None:
+            return results
+
+        x1, y1, x2, y2 = self.crop_bbox
+        img_w, img_h = x2 - x1, y2 - y1
+
+        assert gt_bboxes.shape[-1] == 4
+        gt_bboxes_ = gt_bboxes.copy()
+        gt_bboxes_[..., 0::2] = np.clip(gt_bboxes[..., 0::2] - x1, 0,
+                                        img_w - 1)
+        gt_bboxes_[..., 1::2] = np.clip(gt_bboxes[..., 1::2] - y1, 0,
+                                        img_h - 1)
+        results['gt_bboxes'] = gt_bboxes_
+
+        if proposals is not None:
+            assert proposals.shape[-1] == 4
+            proposals_ = proposals.copy()
+            proposals_[..., 0::2] = np.clip(proposals[..., 0::2] - x1, 0,
+                                            img_w - 1)
+            proposals_[..., 1::2] = np.clip(proposals[..., 1::2] - y1, 0,
+                                            img_h - 1)
+            results['proposals'] = proposals_
+        return results
+
+    def __repr__(self):
+        return f'{self.__class__.__name__}(crop_bbox={self.crop_bbox})'
+
+
+@PIPELINES.register_module()
+class EntityBoxFlip:
+    """Flip the entity boxes and proposals with a probability.
+
+    Reverse the order of elements in the given bounding boxes and proposals
+    with a specific direction. The shape of them are preserved, but the
+    elements are reordered. Only the horizontal flip is supported (seems
+    vertical flipping makes no sense). Required keys are "proposals",
+    "gt_bboxes", added or modified keys are "gt_bboxes". If "proposals"
+    is not None, it will also be modified.
+
+    Args:
+        img_shape (tuple[int]): The img shape.
+    """
+
+    def __init__(self, img_shape):
+        self.img_shape = img_shape
+        assert mmcv.is_tuple_of(img_shape, int)
+
+    def __call__(self, results):
+        proposals = results['proposals']
+        gt_bboxes = results['gt_bboxes']
+        img_h, img_w = self.img_shape
+
+        assert gt_bboxes.shape[-1] == 4
+        gt_bboxes_ = gt_bboxes.copy()
+        gt_bboxes_[..., 0::4] = img_w - gt_bboxes[..., 2::4] - 1
+        gt_bboxes_[..., 2::4] = img_w - gt_bboxes[..., 0::4] - 1
+        if proposals is not None:
+            assert proposals.shape[-1] == 4
+            proposals_ = proposals.copy()
+            proposals_[..., 0::4] = img_w - proposals[..., 2::4] - 1
+            proposals_[..., 2::4] = img_w - proposals[..., 0::4] - 1
+        else:
+            proposals_ = None
+
+        results['proposals'] = proposals_
+        results['gt_bboxes'] = gt_bboxes_
+        return results
+
+    def __repr__(self):
+        repr_str = f'{self.__class__.__name__}(img_shape={self.img_shape})'
+        return repr_str
+
+
+@PIPELINES.register_module()
+class RandomCrop:
     """Vanilla square random crop that specifics the output size.
 
     Required keys in results are "imgs" and "img_shape", added or
@@ -124,10 +318,30 @@ class RandomCrop(object):
         if img_w > self.size:
             x_offset = int(np.random.randint(0, img_w - self.size))
 
+        if 'crop_quadruple' not in results:
+            results['crop_quadruple'] = np.array(
+                [0, 0, 1, 1],  # x, y, w, h
+                dtype=np.float32)
+
+        x_ratio, y_ratio = x_offset / img_w, y_offset / img_h
+        w_ratio, h_ratio = self.size / img_w, self.size / img_h
+
+        old_crop_quadruple = results['crop_quadruple']
+        old_x_ratio, old_y_ratio = old_crop_quadruple[0], old_crop_quadruple[1]
+        old_w_ratio, old_h_ratio = old_crop_quadruple[2], old_crop_quadruple[3]
+        new_crop_quadruple = [
+            old_x_ratio + x_ratio * old_w_ratio,
+            old_y_ratio + y_ratio * old_h_ratio, w_ratio * old_w_ratio,
+            h_ratio * old_x_ratio
+        ]
+        results['crop_quadruple'] = np.array(
+            new_crop_quadruple, dtype=np.float32)
+
         new_h, new_w = self.size, self.size
 
         results['crop_bbox'] = np.array(
             [x_offset, y_offset, x_offset + new_w, y_offset + new_h])
+
         results['img_shape'] = (new_h, new_w)
 
         if not self.lazy:
@@ -152,6 +366,12 @@ class RandomCrop(object):
                                             (lazy_top + bottom)],
                                            dtype=np.float32)
 
+        # Process entity boxes
+        if 'gt_bboxes' in results:
+            assert not self.lazy
+            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
+            results = entity_box_crop(results)
+
         return results
 
     def __repr__(self):
@@ -161,7 +381,7 @@ class RandomCrop(object):
 
 
 @PIPELINES.register_module()
-class RandomResizedCrop(object):
+class RandomResizedCrop:
     """Random crop that specifics the area and height-weight ratio range.
 
     Required keys in results are "imgs", "img_shape", "crop_bbox" and "lazy",
@@ -256,6 +476,25 @@ class RandomResizedCrop(object):
             (img_h, img_w), self.area_range, self.aspect_ratio_range)
         new_h, new_w = bottom - top, right - left
 
+        if 'crop_quadruple' not in results:
+            results['crop_quadruple'] = np.array(
+                [0, 0, 1, 1],  # x, y, w, h
+                dtype=np.float32)
+
+        x_ratio, y_ratio = left / img_w, top / img_h
+        w_ratio, h_ratio = new_w / img_w, new_h / img_h
+
+        old_crop_quadruple = results['crop_quadruple']
+        old_x_ratio, old_y_ratio = old_crop_quadruple[0], old_crop_quadruple[1]
+        old_w_ratio, old_h_ratio = old_crop_quadruple[2], old_crop_quadruple[3]
+        new_crop_quadruple = [
+            old_x_ratio + x_ratio * old_w_ratio,
+            old_y_ratio + y_ratio * old_h_ratio, w_ratio * old_w_ratio,
+            h_ratio * old_x_ratio
+        ]
+        results['crop_quadruple'] = np.array(
+            new_crop_quadruple, dtype=np.float32)
+
         results['crop_bbox'] = np.array([left, top, right, bottom])
         results['img_shape'] = (new_h, new_w)
 
@@ -280,6 +519,10 @@ class RandomResizedCrop(object):
                                             (lazy_top + bottom)],
                                            dtype=np.float32)
 
+        if 'gt_bboxes' in results:
+            assert not self.lazy
+            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
+            results = entity_box_crop(results)
         return results
 
     def __repr__(self):
@@ -291,7 +534,7 @@ class RandomResizedCrop(object):
 
 
 @PIPELINES.register_module()
-class MultiScaleCrop(object):
+class MultiScaleCrop:
     """Crop images with a list of randomly selected scales.
 
     Randomly select the w and h scales from a list of scales. Scale of 1 means
@@ -312,8 +555,8 @@ class MultiScaleCrop(object):
             Default: False.
         num_fixed_crops (int): If set to 5, the cropping bbox will keep 5
             basic fixed regions: "upper left", "upper right", "lower left",
-            "lower right", "center".If set to 13, the cropping bbox will append
-            another 8 fix regions: "center left", "center right",
+            "lower right", "center". If set to 13, the cropping bbox will
+            append another 8 fix regions: "center left", "center right",
             "lower center", "upper center", "upper left quarter",
             "upper right quarter", "lower left quarter", "lower right quarter".
             Default: 5.
@@ -405,6 +648,25 @@ class MultiScaleCrop(object):
         results['img_shape'] = (new_h, new_w)
         results['scales'] = self.scales
 
+        if 'crop_quadruple' not in results:
+            results['crop_quadruple'] = np.array(
+                [0, 0, 1, 1],  # x, y, w, h
+                dtype=np.float32)
+
+        x_ratio, y_ratio = x_offset / img_w, y_offset / img_h
+        w_ratio, h_ratio = new_w / img_w, new_h / img_h
+
+        old_crop_quadruple = results['crop_quadruple']
+        old_x_ratio, old_y_ratio = old_crop_quadruple[0], old_crop_quadruple[1]
+        old_w_ratio, old_h_ratio = old_crop_quadruple[2], old_crop_quadruple[3]
+        new_crop_quadruple = [
+            old_x_ratio + x_ratio * old_w_ratio,
+            old_y_ratio + y_ratio * old_h_ratio, w_ratio * old_w_ratio,
+            h_ratio * old_x_ratio
+        ]
+        results['crop_quadruple'] = np.array(
+            new_crop_quadruple, dtype=np.float32)
+
         if not self.lazy:
             results['imgs'] = [
                 img[y_offset:y_offset + new_h, x_offset:x_offset + new_w]
@@ -427,6 +689,11 @@ class MultiScaleCrop(object):
                                             (lazy_top + bottom)],
                                            dtype=np.float32)
 
+        if 'gt_bboxes' in results:
+            assert not self.lazy
+            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
+            results = entity_box_crop(results)
+
         return results
 
     def __repr__(self):
@@ -440,7 +707,7 @@ class MultiScaleCrop(object):
 
 
 @PIPELINES.register_module()
-class Resize(object):
+class Resize:
     """Resize images to a specific size.
 
     Required keys are "imgs", "img_shape", "modality", added or modified
@@ -523,6 +790,11 @@ class Resize(object):
                 raise NotImplementedError('Put Flip at last for now')
             lazyop['interpolation'] = self.interpolation
 
+        if 'gt_bboxes' in results:
+            assert not self.lazy
+            entity_box_rescale = EntityBoxRescale(self.scale_factor)
+            results = entity_box_rescale(results)
+
         return results
 
     def __repr__(self):
@@ -534,14 +806,69 @@ class Resize(object):
 
 
 @PIPELINES.register_module()
-class Flip(object):
+class RandomRescale:
+    """Randomly resize images so that the short_edge is resized to a specific
+    size in a given range. The scale ratio is unchanged after resizing.
+
+    Required keys are "imgs", "img_shape", "modality", added or modified
+    keys are "imgs", "img_shape", "keep_ratio", "scale_factor", "resize_size",
+    "short_edge".
+
+    Args:
+        scale_range (tuple[int]): The range of short edge length. A closed
+            interval.
+        interpolation (str): Algorithm used for interpolation:
+            "nearest" | "bilinear". Default: "bilinear".
+    """
+
+    def __init__(self, scale_range, interpolation='bilinear'):
+        self.scale_range = scale_range
+        # make sure scale_range is legal, first make sure the type is OK
+        assert mmcv.is_tuple_of(scale_range, int)
+        assert len(scale_range) == 2
+        assert scale_range[0] < scale_range[1]
+        assert np.all([x > 0 for x in scale_range])
+
+        self.keep_ratio = True
+        self.interpolation = interpolation
+
+    def __call__(self, results):
+        """Performs the Resize augmentation.
+
+        Args:
+            results (dict): The resulting dict to be modified and passed
+                to the next transform in pipeline.
+        """
+        short_edge = np.random.randint(self.scale_range[0],
+                                       self.scale_range[1] + 1)
+        resize = Resize((-1, short_edge),
+                        keep_ratio=True,
+                        interpolation=self.interpolation,
+                        lazy=False)
+        results = resize(results)
+
+        results['short_edge'] = short_edge
+        return results
+
+    def __repr__(self):
+        scale_range = self.scale_range
+        repr_str = (f'{self.__class__.__name__}('
+                    f'scale_range=({scale_range[0]}, {scale_range[1]}), '
+                    f'interpolation={self.interpolation})')
+        return repr_str
+
+
+@PIPELINES.register_module()
+class Flip:
     """Flip the input images with a probability.
 
     Reverse the order of elements in the given imgs with a specific direction.
     The shape of the imgs is preserved, but the elements are reordered.
     Required keys are "imgs", "img_shape", "modality", added or modified
     keys are "imgs", "lazy" and "flip_direction". Required keys in "lazy" is
-    None, added or modified key are "flip" and "flip_direction".
+    None, added or modified key are "flip" and "flip_direction". The Flip
+    augmentation should be placed after any cropping / reshaping augmentations,
+    to make sure crop_quadruple is calculated properly.
 
     Args:
         flip_ratio (float): Probability of implementing flip. Default: 0.5.
@@ -571,10 +898,7 @@ class Flip(object):
         if modality == 'Flow':
             assert self.direction == 'horizontal'
 
-        if np.random.rand() < self.flip_ratio:
-            flip = True
-        else:
-            flip = False
+        flip = np.random.rand() < self.flip_ratio
 
         results['flip'] = flip
         results['flip_direction'] = self.direction
@@ -599,6 +923,11 @@ class Flip(object):
             lazyop['flip'] = flip
             lazyop['flip_direction'] = self.direction
 
+        if 'gt_bboxes' in results and flip:
+            assert not self.lazy and self.direction == 'horizontal'
+            entity_box_flip = EntityBoxFlip(results['img_shape'])
+            results = entity_box_flip(results)
+
         return results
 
     def __repr__(self):
@@ -610,7 +939,7 @@ class Flip(object):
 
 
 @PIPELINES.register_module()
-class Normalize(object):
+class Normalize:
     """Normalize images with the given mean and std value.
 
     Required keys are "imgs", "img_shape", "modality", added or modified
@@ -658,7 +987,7 @@ class Normalize(object):
             results['img_norm_cfg'] = dict(
                 mean=self.mean, std=self.std, to_bgr=self.to_bgr)
             return results
-        elif modality == 'Flow':
+        if modality == 'Flow':
             num_imgs = len(results['imgs'])
             assert num_imgs % 2 == 0
             assert self.mean.shape[0] == 2
@@ -684,8 +1013,7 @@ class Normalize(object):
                 adjust_magnitude=self.adjust_magnitude)
             results['img_norm_cfg'] = args
             return results
-        else:
-            raise NotImplementedError
+        raise NotImplementedError
 
     def __repr__(self):
         repr_str = (f'{self.__class__.__name__}('
@@ -697,7 +1025,7 @@ class Normalize(object):
 
 
 @PIPELINES.register_module()
-class ColorJitter(object):
+class ColorJitter:
     """Randomly distort the brightness, contrast, saturation and hue of images,
     and add PCA based noise into images.
 
@@ -879,7 +1207,7 @@ class ColorJitter(object):
 
 
 @PIPELINES.register_module()
-class CenterCrop(object):
+class CenterCrop:
     """Crop the center area from images.
 
     Required keys are "imgs", "img_shape", added or modified keys are "imgs",
@@ -919,6 +1247,25 @@ class CenterCrop(object):
         results['crop_bbox'] = np.array([left, top, right, bottom])
         results['img_shape'] = (new_h, new_w)
 
+        if 'crop_quadruple' not in results:
+            results['crop_quadruple'] = np.array(
+                [0, 0, 1, 1],  # x, y, w, h
+                dtype=np.float32)
+
+        x_ratio, y_ratio = left / img_w, top / img_h
+        w_ratio, h_ratio = new_w / img_w, new_h / img_h
+
+        old_crop_quadruple = results['crop_quadruple']
+        old_x_ratio, old_y_ratio = old_crop_quadruple[0], old_crop_quadruple[1]
+        old_w_ratio, old_h_ratio = old_crop_quadruple[2], old_crop_quadruple[3]
+        new_crop_quadruple = [
+            old_x_ratio + x_ratio * old_w_ratio,
+            old_y_ratio + y_ratio * old_h_ratio, w_ratio * old_w_ratio,
+            h_ratio * old_x_ratio
+        ]
+        results['crop_quadruple'] = np.array(
+            new_crop_quadruple, dtype=np.float32)
+
         if not self.lazy:
             results['imgs'] = [
                 img[top:bottom, left:right] for img in results['imgs']
@@ -940,6 +1287,11 @@ class CenterCrop(object):
                                             (lazy_top + bottom)],
                                            dtype=np.float32)
 
+        if 'gt_bboxes' in results:
+            assert not self.lazy
+            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
+            results = entity_box_crop(results)
+
         return results
 
     def __repr__(self):
@@ -949,7 +1301,7 @@ class CenterCrop(object):
 
 
 @PIPELINES.register_module()
-class ThreeCrop(object):
+class ThreeCrop:
     """Crop images into three crops.
 
     Crop the images equally into three crops with equal intervals along the
@@ -1020,7 +1372,7 @@ class ThreeCrop(object):
 
 
 @PIPELINES.register_module()
-class TenCrop(object):
+class TenCrop:
     """Crop the images into 10 crops (corner + center + flip).
 
     Crop the four corners and the center part of the image with the same
@@ -1089,7 +1441,7 @@ class TenCrop(object):
 
 
 @PIPELINES.register_module()
-class MultiGroupCrop(object):
+class MultiGroupCrop:
     """Randomly crop the images into several groups.
 
     Crop the random region with the same given crop_size and bounding box
@@ -1106,9 +1458,8 @@ class MultiGroupCrop(object):
         self.crop_size = _pair(crop_size)
         self.groups = groups
         if not mmcv.is_tuple_of(self.crop_size, int):
-            raise TypeError(
-                'Crop size must be int or tuple of int, but got {}'.format(
-                    type(crop_size)))
+            raise TypeError('Crop size must be int or tuple of int, '
+                            f'but got {type(crop_size)}')
 
         if not isinstance(groups, int):
             raise TypeError(f'Groups must be int, but got {type(groups)}.')
@@ -1153,3 +1504,114 @@ class MultiGroupCrop(object):
                     f'(crop_size={self.crop_size}, '
                     f'groups={self.groups})')
         return repr_str
+
+
+@PIPELINES.register_module()
+class AudioAmplify:
+    """Amplify the waveform.
+
+    Required keys are "audios", added or modified keys are "audios",
+    "amplify_ratio".
+
+    Args:
+        ratio (float): The ratio used to amplify the audio waveform.
+    """
+
+    def __init__(self, ratio):
+        if isinstance(ratio, float):
+            self.ratio = ratio
+        else:
+            raise TypeError('Amplification ratio should be float.')
+
+    def __call__(self, results):
+        """Perfrom the audio amplification.
+
+        Args:
+            results (dict): The resulting dict to be modified and passed
+                to the next transform in pipeline.
+        """
+
+        assert 'audios' in results
+        results['audios'] *= self.ratio
+        results['amplify_ratio'] = self.ratio
+
+        return results
+
+    def __repr__(self):
+        repr_str = f'{self.__class__.__name__}(ratio={self.ratio})'
+        return repr_str
+
+
+@PIPELINES.register_module()
+class MelSpectrogram:
+    """MelSpectrogram. Transfer an audio wave into a melspectogram figure.
+
+    Required keys are "audios", "sample_rate", "num_clips", added or modified
+    keys are "audios".
+
+    Args:
+        window_size (int): The window size in milisecond. Default: 32.
+        step_size (int): The step size in milisecond. Default: 16.
+        n_mels (int): Number of mels. Default: 80.
+        fixed_length (int): The sample length of melspectrogram maybe not
+            exactly as wished due to different fps, fix the length for batch
+            collation by truncating or padding. Default: 128.
+    """
+
+    def __init__(self,
+                 window_size=32,
+                 step_size=16,
+                 n_mels=80,
+                 fixed_length=128):
+        if all(
+                isinstance(x, int)
+                for x in [window_size, step_size, n_mels, fixed_length]):
+            self.window_size = window_size
+            self.step_size = step_size
+            self.n_mels = n_mels
+            self.fixed_length = fixed_length
+        else:
+            raise TypeError('All arguments should be int.')
+
+    def __call__(self, results):
+        """Perform MelSpectrogram transformation.
+
+        Args:
+            results (dict): The resulting dict to be modified and passed
+                to the next transform in pipeline.
+        """
+        try:
+            import librosa
+        except ImportError:
+            raise ImportError('Install librosa first.')
+        signals = results['audios']
+        sample_rate = results['sample_rate']
+        n_fft = int(round(sample_rate * self.window_size / 1000))
+        hop_length = int(round(sample_rate * self.step_size / 1000))
+        melspectrograms = list()
+        for clip_idx in range(results['num_clips']):
+            clip_signal = signals[clip_idx]
+            mel = librosa.feature.melspectrogram(
+                y=clip_signal,
+                sr=sample_rate,
+                n_fft=n_fft,
+                hop_length=hop_length,
+                n_mels=self.n_mels)
+            if mel.shape[0] >= self.fixed_length:
+                mel = mel[:self.fixed_length, :]
+            else:
+                mel = np.pad(
+                    mel, ((0, mel.shape[-1] - self.fixed_length), (0, 0)),
+                    mode='edge')
+            melspectrograms.append(mel)
+
+        results['audios'] = np.array(melspectrograms)
+        return results
+
+    def __repr__(self):
+        repr_str = (f'{self.__class__.__name__}'
+                    f'(window_size={self.window_size}), '
+                    f'step_size={self.step_size}, '
+                    f'n_mels={self.n_mels}, '
+                    f'fixed_length={self.fixed_length})')
+        return repr_str
