@@ -16,20 +16,19 @@ def to_tensor(data):
     """
     if isinstance(data, torch.Tensor):
         return data
-    elif isinstance(data, np.ndarray):
+    if isinstance(data, np.ndarray):
         return torch.from_numpy(data)
-    elif isinstance(data, Sequence) and not mmcv.is_str(data):
+    if isinstance(data, Sequence) and not mmcv.is_str(data):
         return torch.tensor(data)
-    elif isinstance(data, int):
+    if isinstance(data, int):
         return torch.LongTensor([data])
-    elif isinstance(data, float):
+    if isinstance(data, float):
         return torch.FloatTensor([data])
-    else:
-        raise TypeError(f'type {type(data)} cannot be converted to tensor.')
+    raise TypeError(f'type {type(data)} cannot be converted to tensor.')
 
 
 @PIPELINES.register_module()
-class ToTensor(object):
+class ToTensor:
     """Convert some values in results dict to `torch.Tensor` type in data
     loader pipeline.
 
@@ -56,13 +55,40 @@ class ToTensor(object):
 
 
 @PIPELINES.register_module()
-class ToDataContainer(object):
+class Rename:
+    """Rename the key in results.
+
+    Args:
+        mapping (dict): The keys in results that need to be renamed. The key of
+            the dict is the original name, while the value is the new name. If
+            the original name not found in results, do nothing.
+            Default: dict().
+    """
+
+    def __init__(self, mapping):
+        self.mapping = mapping
+
+    def __call__(self, results):
+        for key, value in self.mapping.items():
+            if key in results:
+                assert isinstance(key, str) and isinstance(value, str)
+                assert value not in results, ('the new name already exists in '
+                                              'results')
+                results[value] = results[key]
+                results.pop(key)
+        return results
+
+
+@PIPELINES.register_module()
+class ToDataContainer:
     """Convert the data to DataContainer.
 
     Args:
         fields (Sequence[dict]): Required fields to be converted
             with keys and attributes. E.g.
             fields=(dict(key='gt_bbox', stack=False),).
+            Note that key can also be a list of keys, if so, every tensor in
+            the list will be converted to DataContainer.
     """
 
     def __init__(self, fields):
@@ -78,7 +104,11 @@ class ToDataContainer(object):
         for field in self.fields:
             _field = field.copy()
             key = _field.pop('key')
-            results[key] = DC(results[key], **_field)
+            if isinstance(key, list):
+                for item in key:
+                    results[item] = DC(results[item], **_field)
+            else:
+                results[key] = DC(results[key], **_field)
         return results
 
     def __repr__(self):
@@ -86,7 +116,7 @@ class ToDataContainer(object):
 
 
 @PIPELINES.register_module()
-class ImageToTensor(object):
+class ImageToTensor:
     """Convert image type to `torch.Tensor` type.
 
     Args:
@@ -112,7 +142,7 @@ class ImageToTensor(object):
 
 
 @PIPELINES.register_module()
-class Transpose(object):
+class Transpose:
     """Transpose image channels to a given order.
 
     Args:
@@ -125,7 +155,7 @@ class Transpose(object):
         self.order = order
 
     def __call__(self, results):
-        """Performs the Transpose formating.
+        """Performs the Transpose formatting.
 
         Args:
             results (dict): The resulting dict to be modified and passed
@@ -141,21 +171,21 @@ class Transpose(object):
 
 
 @PIPELINES.register_module()
-class Collect(object):
+class Collect:
     """Collect data from the loader relevant to the specific task.
 
     This keeps the items in ``keys`` as it is, and collect items in
     ``meta_keys`` into a meta item called ``meta_name``.This is usually
     the last stage of the data loader pipeline.
     For example, when keys='imgs', meta_keys=('filename', 'label',
-    'original_shape'), meta_name='img_meta', the results will be a dict with
-    keys 'imgs' and 'img_meta', where 'img_meta' is a DataContainer of another
-    dict with keys 'filename', 'label', 'original_shape'.
+    'original_shape'), meta_name='img_metas', the results will be a dict with
+    keys 'imgs' and 'img_metas', where 'img_metas' is a DataContainer of
+    another dict with keys 'filename', 'label', 'original_shape'.
 
     Args:
         keys (Sequence[str]): Required keys to be collected.
         meta_name (str): The name of the key that contains meta infomation.
-            This key is always populated. Default: "img_meta".
+            This key is always populated. Default: "img_metas".
         meta_keys (Sequence[str]): Keys that are collected under meta_name.
             The contents of the ``meta_name`` dictionary depends on
             ``meta_keys``.
@@ -175,16 +205,20 @@ class Collect(object):
                 - mean - per channel mean subtraction
                 - std - per channel std divisor
                 - to_rgb - bool indicating if bgr was converted to rgb
+        nested (bool): If set as True, will apply data[x] = [data[x]] to all
+            items in data. The arg is added for compatibility. Default: False.
     """
 
     def __init__(self,
                  keys,
                  meta_keys=('filename', 'label', 'original_shape', 'img_shape',
                             'pad_shape', 'flip_direction', 'img_norm_cfg'),
-                 meta_name='img_meta'):
+                 meta_name='img_metas',
+                 nested=False):
         self.keys = keys
         self.meta_keys = meta_keys
         self.meta_name = meta_name
+        self.nested = nested
 
     def __call__(self, results):
         """Performs the Collect formating.
@@ -202,16 +236,20 @@ class Collect(object):
             for key in self.meta_keys:
                 meta[key] = results[key]
             data[self.meta_name] = DC(meta, cpu_only=True)
+        if self.nested:
+            for k in data:
+                data[k] = [data[k]]
 
         return data
 
     def __repr__(self):
         return (f'{self.__class__.__name__}('
-                f'keys={self.keys}, meta_keys={self.meta_keys})')
+                f'keys={self.keys}, meta_keys={self.meta_keys}, '
+                f'nested={self.nested})')
 
 
 @PIPELINES.register_module()
-class FormatShape(object):
+class FormatShape:
     """Format final imgs shape to the given input_format.
 
     Required keys are "imgs", "num_clips" and "clip_len", added or modified
@@ -219,10 +257,14 @@ class FormatShape(object):
 
     Args:
         input_format (str): Define the final imgs format.
+        collapse (bool): To collpase input_format N... to ... (NCTHW to CTHW,
+            etc.) if N is 1. Should be set as True when training and testing
+            detectors. Default: False.
     """
 
-    def __init__(self, input_format):
+    def __init__(self, input_format, collapse=False):
         self.input_format = input_format
+        self.collapse = collapse
         if self.input_format not in ['NCTHW', 'NCHW', 'NCHW_Flow', 'NPTCHW']:
             raise ValueError(
                 f'The input format {self.input_format} is invalid.')
@@ -237,6 +279,9 @@ class FormatShape(object):
         imgs = results['imgs']
         # [M x H x W x C]
         # M = 1 * N_crops * N_clips * L
+        if self.collapse:
+            assert results['num_clips'] == 1
+
         if self.input_format == 'NCTHW':
             num_clips = results['num_clips']
             clip_len = results['clip_len']
@@ -273,6 +318,9 @@ class FormatShape(object):
             # M = N_clips x L
             imgs = np.transpose(imgs, (0, 1, 4, 2, 3))
             # P x M x C x H x W
+        if self.collapse:
+            assert imgs.shape[0] == 1
+            imgs = imgs.squeeze(0)
 
         results['imgs'] = imgs
         results['input_shape'] = imgs.shape
@@ -282,3 +330,41 @@ class FormatShape(object):
         repr_str = self.__class__.__name__
         repr_str += f"(input_format='{self.input_format}')"
         return repr_str
+
+
+@PIPELINES.register_module()
+class FormatAudioShape:
+    """Format final audio shape to the given input_format.
+
+    Required keys are "imgs", "num_clips" and "clip_len", added or modified
+    keys are "imgs" and "input_shape".
+
+    Args:
+        input_format (str): Define the final imgs format.
+    """
+
+    def __init__(self, input_format):
+        self.input_format = input_format
+        if self.input_format not in ['NCTF']:
+            raise ValueError(
+                f'The input format {self.input_format} is invalid.')
+
+    def __call__(self, results):
+        """Performs the FormatShape formatting.
+
+        Args:
+            results (dict): The resulting dict to be modified and passed
+                to the next transform in pipeline.
+        """
+        audios = results['audios']
+        # clip x sample x freq -> clip x channel x sample x freq
+        clip, sample, freq = audios.shape
+        audios = audios.reshape(clip, 1, sample, freq)
+        results['audios'] = audios
+        results['input_shape'] = audios.shape
+        return results
+
+    def __repr__(self):
+        repr_str = self.__class__.__name__
+        repr_str += f"(input_format='{self.input_format}')"
+        return repr_str
