@@ -1,5 +1,6 @@
 import copy
 import os.path as osp
+import warnings
 
 import mmcv
 import numpy as np
@@ -20,7 +21,7 @@ class SSNInstance:
         start_frame (int): Index of the proposal's start frame.
         end_frame (int): Index of the proposal's end frame.
         num_video_frames (int): Total frames of the video.
-        label (int): The category label of the proposal. Default: None.
+        label (int | None): The category label of the proposal. Default: None.
         best_iou (float): The highest IOU with the groundtruth instance.
             Default: 0.
         overlap_self (float): Percent of the proposal's own span contained
@@ -45,13 +46,11 @@ class SSNInstance:
         self.size_reg = None
         self.regression_targets = [0., 0.]
 
-    def compute_regression_targets(self, gt_list, positive_threshold):
+    def compute_regression_targets(self, gt_list):
         """Compute regression targets of positive proposals.
 
         Args:
             gt_list (list): The list of groundtruth instances.
-            positive_threshold (float): Minimum threshold of overlap of
-                positive/foreground proposals and groundtruths.
         """
         # Find the groundtruth instance with the highest IOU.
         ious = [
@@ -82,7 +81,7 @@ class SSNInstance:
 class SSNDataset(BaseDataset):
     """Proposal frame dataset for Structured Segment Networks.
 
-    Based on proposal information, the dataset loads raw frames and apply
+    Based on proposal information, the dataset loads raw frames and applies
     specified transforms to return a dict containing the frame tensors and
     other information.
 
@@ -327,45 +326,38 @@ class SSNDataset(BaseDataset):
                     proposals=proposals))
         return video_infos
 
-    def results_to_detections(self,
-                              results,
-                              top_k=2000,
-                              softmax_before_filter=True,
-                              cls_top_k=2,
-                              **kwargs):
+    def results_to_detections(self, results, top_k=2000, **kwargs):
         """Convert prediction results into detections.
 
         Args:
             results (list): Prediction results.
             top_k (int): Number of top results. Default: 2000.
-            softmax_before_filter (bool): Whether to perform softmax operations
-                before filtering results. Default: True.
-            cls_top_k (int): Number of top results for each class. Default: 2.
 
         Returns:
             list: Detection results.
         """
-        num_classes = results[0][1].shape[1] - 1
+        num_classes = results[0]['activity_scores'].shape[1] - 1
         detections = [dict() for _ in range(num_classes)]
 
         for idx in range(len(self)):
             video_id = self.video_infos[idx]['video_id']
-            relative_proposals = results[idx][0]
+            relative_proposals = results[idx]['relative_proposal_list']
             if len(relative_proposals[0].shape) == 3:
                 relative_proposals = np.squeeze(relative_proposals, 0)
 
-            action_scores = results[idx][1]
-            complete_scores = results[idx][2]
-            regression_scores = results[idx][3]
+            activity_scores = results[idx]['activity_scores']
+            completeness_scores = results[idx]['completeness_scores']
+            regression_scores = results[idx]['bbox_preds']
             if regression_scores is None:
                 regression_scores = np.zeros(
-                    len(relative_proposals), num_classes, 2, dtype=np.float32)
+                    (len(relative_proposals), num_classes, 2),
+                    dtype=np.float32)
             regression_scores = regression_scores.reshape((-1, num_classes, 2))
 
             if top_k <= 0:
                 combined_scores = (
-                    softmax(action_scores[:, 1:], dim=1) *
-                    np.exp(complete_scores))
+                    softmax(activity_scores[:, 1:], dim=1) *
+                    np.exp(completeness_scores))
                 for i in range(num_classes):
                     center_scores = regression_scores[:, i, 0][:, None]
                     duration_scores = regression_scores[:, i, 1][:, None]
@@ -375,8 +367,8 @@ class SSNDataset(BaseDataset):
                         axis=1)
             else:
                 combined_scores = (
-                    softmax(action_scores[:, 1:], dim=1) *
-                    np.exp(complete_scores))
+                    softmax(activity_scores[:, 1:], dim=1) *
+                    np.exp(completeness_scores))
                 keep_idx = np.argsort(combined_scores.ravel())[-top_k:]
                 for k in keep_idx:
                     class_idx = k % num_classes
@@ -401,19 +393,37 @@ class SSNDataset(BaseDataset):
     def evaluate(self,
                  results,
                  metrics='mAP',
-                 eval_dataset='thumos14',
-                 **kwargs):
+                 metric_options=dict(mAP=dict(eval_dataset='thumos14')),
+                 logger=None,
+                 **deprecated_kwargs):
         """Evaluation in SSN proposal dataset.
 
         Args:
             results (list[dict]): Output results.
             metrics (str | sequence[str]): Metrics to be performed.
                 Defaults: 'mAP'.
-            eval_dataset (str): Dataset to be evaluated.
+            metric_options (dict): Dict for metric options. Options are
+                ``eval_dataset`` for ``mAP``.
+                Default: ``dict(mAP=dict(eval_dataset='thumos14'))``.
+            logger (logging.Logger | None): Logger for recording.
+                Default: None.
+            deprecated_kwargs (dict): Used for containing deprecated arguments.
+                See 'https://github.com/open-mmlab/mmaction2/pull/286'.
 
         Returns:
             dict: Evaluation results for evaluation metrics.
         """
+        # Protect ``metric_options`` since it uses mutable value as default
+        metric_options = copy.deepcopy(metric_options)
+
+        if deprecated_kwargs != {}:
+            warnings.warn(
+                'Option arguments for metrics has been changed to '
+                "`metric_options`, See 'https://github.com/open-mmlab/mmaction2/pull/286' "  # noqa: E501
+                'for more details')
+            metric_options['mAP'] = dict(metric_options['mAP'],
+                                         **deprecated_kwargs)
+
         if not isinstance(results, list):
             raise TypeError(f'results must be a list, but got {type(results)}')
         assert len(results) == len(self), (
@@ -430,7 +440,7 @@ class SSNDataset(BaseDataset):
 
         if self.use_regression:
             self.logger.info('Performing location regression')
-            for class_idx in range(len(detections)):
+            for class_idx, _ in enumerate(detections):
                 detections[class_idx] = {
                     k: perform_regression(v)
                     for k, v in detections[class_idx].items()
@@ -438,7 +448,7 @@ class SSNDataset(BaseDataset):
             self.logger.info('Regression finished')
 
         self.logger.info('Performing NMS')
-        for class_idx in range(len(detections)):
+        for class_idx, _ in enumerate(detections):
             detections[class_idx] = {
                 k: temporal_nms(v, self.evaluater.nms)
                 for k, v in detections[class_idx].items()
@@ -447,13 +457,13 @@ class SSNDataset(BaseDataset):
 
         # get gts
         all_gts = self.get_all_gts()
-        for class_idx in range(len(detections)):
-            if (class_idx not in all_gts):
+        for class_idx, _ in enumerate(detections):
+            if class_idx not in all_gts:
                 all_gts[class_idx] = dict()
 
         # get predictions
         plain_detections = {}
-        for class_idx in range(len(detections)):
+        for class_idx, _ in enumerate(detections):
             detection_list = []
             for video, dets in detections[class_idx].items():
                 detection_list.extend([[video, class_idx] + x[:3]
@@ -463,6 +473,8 @@ class SSNDataset(BaseDataset):
         eval_results = {}
         for metric in metrics:
             if metric == 'mAP':
+                eval_dataset = metric_options.setdefault('mAP', {}).setdefault(
+                    'eval_dataset', 'thumos14')
                 if eval_dataset == 'thumos14':
                     iou_range = np.arange(0.1, 1.0, .1)
                     ap_values = eval_ap(plain_detections, all_gts, iou_range)
@@ -513,7 +525,8 @@ class SSNDataset(BaseDataset):
 
         return gts
 
-    def get_positives(self, gts, proposals, positive_threshold, with_gt=True):
+    @staticmethod
+    def get_positives(gts, proposals, positive_threshold, with_gt=True):
         """Get positive/foreground proposals.
 
         Args:
@@ -537,12 +550,12 @@ class SSNDataset(BaseDataset):
             positives.extend(gts)
 
         for proposal in positives:
-            proposal.compute_regression_targets(gts, positive_threshold)
+            proposal.compute_regression_targets(gts)
 
         return positives
 
-    def get_negatives(self,
-                      proposals,
+    @staticmethod
+    def get_negatives(proposals,
                       incomplete_iou_threshold,
                       background_iou_threshold,
                       background_coverage_threshold=0.01,
@@ -557,8 +570,7 @@ class SSNDataset(BaseDataset):
             background_iou_threshold (float): Maximum threshold of overlap
                 of background proposals and groundtruths.
             background_coverage_threshold (float): Minimum coverage
-                of background proposals in video duration.
-                Default: 0.01.
+                of background proposals in video duration. Default: 0.01.
             incomplete_overlap_threshold (float): Minimum percent of incomplete
                 proposals' own span contained in a groundtruth instance.
                 Default: 0.7.
@@ -587,8 +599,8 @@ class SSNDataset(BaseDataset):
         Args:
             record (dict): Information of the video instance(video_info[idx]).
                 key: frame_dir, video_id, total_frames,
-                     gts: List of groundtruth instances(:obj:`SSNInstance`).
-                     proposals: List of proposal instances(:obj:`SSNInstance`).
+                gts: List of groundtruth instances(:obj:`SSNInstance`).
+                proposals: List of proposal instances(:obj:`SSNInstance`).
         """
         positives = self.get_positives(record['gts'], record['proposals'],
                                        self.assigner.positive_iou_threshold,
@@ -628,14 +640,11 @@ class SSNDataset(BaseDataset):
                 idx = np.random.choice(
                     len(dataset_pool), num_requested_proposals, replace=False)
                 return [(dataset_pool[x], proposal_type) for x in idx]
-            else:
-                replicate = len(video_pool) < num_requested_proposals
-                idx = np.random.choice(
-                    len(video_pool),
-                    num_requested_proposals,
-                    replace=replicate)
-                return [((video_id, video_pool[x]), proposal_type)
-                        for x in idx]
+
+            replicate = len(video_pool) < num_requested_proposals
+            idx = np.random.choice(
+                len(video_pool), num_requested_proposals, replace=replicate)
+            return [((video_id, video_pool[x]), proposal_type) for x in idx]
 
         out_proposals = []
         out_proposals.extend(
@@ -762,7 +771,7 @@ class SSNDataset(BaseDataset):
             num_frames = proposal[0][1].num_video_frames
 
             (starting_scale_factor, ending_scale_factor,
-             stage_split) = self._get_stage(proposal[0][1], num_frames)
+             _) = self._get_stage(proposal[0][1], num_frames)
 
             # proposal[1]: Type id of proposal.
             # Positive/Foreground: 0
