@@ -1,3 +1,5 @@
+import warnings
+
 import torch.nn as nn
 import torch.utils.checkpoint as cp
 from mmcv.cnn import (ConvModule, NonLocal3d, build_activation_layer,
@@ -9,6 +11,12 @@ from torch.nn.modules.utils import _ntuple, _triple
 from ...utils import get_root_logger
 from ..registry import BACKBONES
 
+try:
+    import mmdet  # noqa
+    from mmdet.models.builder import SHARED_HEADS as MMDET_SHARED_HEADS
+except (ImportError, ModuleNotFoundError):
+    warnings.warn('Please install mmdet to use MMDET_SHARED_HEADS')
+
 
 class BasicBlock3d(nn.Module):
     """BasicBlock 3d block for ResNet3D.
@@ -24,9 +32,6 @@ class BasicBlock3d(nn.Module):
             stride-two layer is the 3x3 conv layer, otherwise the stride-two
             layer is the first 1x1 conv layer. Default: 'pytorch'.
         inflate (bool): Whether to inflate kernel. Default: True.
-        inflate_style (str): ``3x1x1`` or ``1x1x1``. which determines the
-            kernel sizes and padding strides for conv1 and conv2 in each block.
-            Default: '3x1x1'.
         non_local (bool): Determine whether to apply non-local module in this
             block. Default: False.
         non_local_cfg (dict): Config for non-local module. Default: ``dict()``.
@@ -50,16 +55,17 @@ class BasicBlock3d(nn.Module):
                  downsample=None,
                  style='pytorch',
                  inflate=True,
-                 inflate_style='3x1x1',
                  non_local=False,
                  non_local_cfg=dict(),
                  conv_cfg=dict(type='Conv3d'),
                  norm_cfg=dict(type='BN3d'),
                  act_cfg=dict(type='ReLU'),
-                 with_cp=False):
+                 with_cp=False,
+                 **kwargs):
         super().__init__()
         assert style in ['pytorch', 'caffe']
-        assert inflate_style in ['3x1x1', '3x3x3']
+        # make sure that only ``inflate_style`` is passed into kwargs
+        assert set(kwargs.keys()).issubset(['inflate_style'])
 
         self.inplanes = inplanes
         self.planes = planes
@@ -68,7 +74,6 @@ class BasicBlock3d(nn.Module):
         self.dilation = dilation
         self.style = style
         self.inflate = inflate
-        self.inflate_style = inflate_style
         self.conv_cfg = conv_cfg
         self.norm_cfg = norm_cfg
         self.act_cfg = act_cfg
@@ -166,7 +171,7 @@ class Bottleneck3d(nn.Module):
             stride-two layer is the 3x3 conv layer, otherwise the stride-two
             layer is the first 1x1 conv layer. Default: 'pytorch'.
         inflate (bool): Whether to inflate kernel. Default: True.
-        inflate_style (str): ``3x1x1`` or ``1x1x1``. which determines the
+        inflate_style (str): ``3x1x1`` or ``3x3x3``. which determines the
             kernel sizes and padding strides for conv1 and conv2 in each block.
             Default: '3x1x1'.
         non_local (bool): Determine whether to apply non-local module in this
@@ -420,7 +425,7 @@ class ResNet3d(nn.Module):
         self.in_channels = in_channels
         self.base_channels = base_channels
         self.num_stages = num_stages
-        assert num_stages >= 1 and num_stages <= 4
+        assert 1 <= num_stages <= 4
         self.out_indices = out_indices
         assert max(out_indices) < num_stages
         self.spatial_strides = spatial_strides
@@ -484,8 +489,8 @@ class ResNet3d(nn.Module):
         self.feat_dim = self.block.expansion * self.base_channels * 2**(
             len(self.stage_blocks) - 1)
 
-    def make_res_layer(self,
-                       block,
+    @staticmethod
+    def make_res_layer(block,
                        inplanes,
                        planes,
                        blocks,
@@ -598,7 +603,8 @@ class ResNet3d(nn.Module):
 
         return nn.Sequential(*layers)
 
-    def _inflate_conv_params(self, conv3d, state_dict_2d, module_name_2d,
+    @staticmethod
+    def _inflate_conv_params(conv3d, state_dict_2d, module_name_2d,
                              inflated_param_names):
         """Inflate a conv module from 2d to 3d.
 
@@ -625,7 +631,8 @@ class ResNet3d(nn.Module):
             conv3d.bias.data.copy_(state_dict_2d[bias_2d_name])
             inflated_param_names.append(bias_2d_name)
 
-    def _inflate_bn_params(self, bn3d, state_dict_2d, module_name_2d,
+    @staticmethod
+    def _inflate_bn_params(bn3d, state_dict_2d, module_name_2d,
                            inflated_param_names):
         """Inflate a norm module from 2d to 3d.
 
@@ -652,7 +659,8 @@ class ResNet3d(nn.Module):
                 param.data.copy_(param_2d)
                 inflated_param_names.append(param_2d_name)
 
-    def inflate_weights(self, logger):
+    @staticmethod
+    def _inflate_weights(self, logger):
         """Inflate the resnet2d parameters to resnet3d.
 
         The differences between resnet3d and resnet2d mainly lie in an extra
@@ -716,6 +724,9 @@ class ResNet3d(nn.Module):
             logger.info(f'These parameters in the 2d checkpoint are not loaded'
                         f': {remaining_names}')
 
+    def inflate_weights(self, logger):
+        self._inflate_weights(self, logger)
+
     def _make_stem_layer(self):
         """Construct the stem layers consists of a conv+norm+act module and a
         pooling layer."""
@@ -751,9 +762,18 @@ class ResNet3d(nn.Module):
             for param in m.parameters():
                 param.requires_grad = False
 
-    def init_weights(self):
+    @staticmethod
+    def _init_weights(self, pretrained=None):
         """Initiate the parameters either from existing checkpoint or from
-        scratch."""
+        scratch.
+
+        Args:
+            pretrained (str | None): The path of the pretrained weight. Will
+                override the original `pretrained` if set. The arg is added to
+                be compatible with mmdet. Default: None.
+        """
+        if pretrained:
+            self.pretrained = pretrained
         if isinstance(self.pretrained, str):
             logger = get_root_logger()
             logger.info(f'load model from: {self.pretrained}')
@@ -783,6 +803,9 @@ class ResNet3d(nn.Module):
         else:
             raise TypeError('pretrained must be a str or None')
 
+    def init_weights(self, pretrained=None):
+        self._init_weights(self, pretrained)
+
     def forward(self, x):
         """Defines the computation performed at every call.
 
@@ -805,8 +828,8 @@ class ResNet3d(nn.Module):
                 outs.append(x)
         if len(outs) == 1:
             return outs[0]
-        else:
-            return tuple(outs)
+
+        return tuple(outs)
 
     def train(self, mode=True):
         """Set the optimization status when training."""
@@ -816,3 +839,165 @@ class ResNet3d(nn.Module):
             for m in self.modules():
                 if isinstance(m, _BatchNorm):
                     m.eval()
+
+
+@BACKBONES.register_module()
+class ResNet3dLayer(nn.Module):
+    """ResNet 3d Layer.
+
+    Args:
+        depth (int): Depth of resnet, from {18, 34, 50, 101, 152}.
+        pretrained (str | None): Name of pretrained model.
+        pretrained2d (bool): Whether to load pretrained 2D model.
+            Default: True.
+        stage (int): The index of Resnet stage. Default: 3.
+        base_channels (int): Channel num of stem output features. Default: 64.
+        spatial_stride (int): The 1st res block's spatial stride. Default 2.
+        temporal_stride (int): The 1st res block's temporal stride. Default 1.
+        dilation (int): The dilation. Default: 1.
+        style (str): `pytorch` or `caffe`. If set to "pytorch", the stride-two
+            layer is the 3x3 conv layer, otherwise the stride-two layer is
+            the first 1x1 conv layer. Default: 'pytorch'.
+        all_frozen (bool): Frozen all modules in the layer. Default: False.
+        inflate (int): Inflate Dims of each block. Default: 1.
+        inflate_style (str): ``3x1x1`` or ``1x1x1``. which determines the
+            kernel sizes and padding strides for conv1 and conv2 in each block.
+            Default: '3x1x1'.
+        conv_cfg (dict): Config for conv layers. required keys are ``type``
+            Default: ``dict(type='Conv3d')``.
+        norm_cfg (dict): Config for norm layers. required keys are ``type`` and
+            ``requires_grad``.
+            Default: ``dict(type='BN3d', requires_grad=True)``.
+        act_cfg (dict): Config dict for activation layer.
+            Default: ``dict(type='ReLU', inplace=True)``.
+        norm_eval (bool): Whether to set BN layers to eval mode, namely, freeze
+            running stats (mean and var). Default: False.
+        with_cp (bool): Use checkpoint or not. Using checkpoint will save some
+            memory while slowing down the training speed. Default: False.
+        zero_init_residual (bool):
+            Whether to use zero initialization for residual block,
+            Default: True.
+        kwargs (dict, optional): Key arguments for "make_res_layer".
+    """
+
+    def __init__(self,
+                 depth,
+                 pretrained,
+                 pretrained2d=True,
+                 stage=3,
+                 base_channels=64,
+                 spatial_stride=2,
+                 temporal_stride=1,
+                 dilation=1,
+                 style='pytorch',
+                 all_frozen=False,
+                 inflate=1,
+                 inflate_style='3x1x1',
+                 conv_cfg=dict(type='Conv3d'),
+                 norm_cfg=dict(type='BN3d', requires_grad=True),
+                 act_cfg=dict(type='ReLU', inplace=True),
+                 norm_eval=False,
+                 with_cp=False,
+                 zero_init_residual=True,
+                 **kwargs):
+
+        super().__init__()
+        self.arch_settings = ResNet3d.arch_settings
+        assert depth in self.arch_settings
+
+        self.make_res_layer = ResNet3d.make_res_layer
+        self._inflate_conv_params = ResNet3d._inflate_conv_params
+        self._inflate_bn_params = ResNet3d._inflate_bn_params
+        self._inflate_weights = ResNet3d._inflate_weights
+        self._init_weights = ResNet3d._init_weights
+
+        self.depth = depth
+        self.pretrained = pretrained
+        self.pretrained2d = pretrained2d
+        self.stage = stage
+        # stage index is 0 based
+        assert stage >= 0 and stage <= 3
+        self.base_channels = base_channels
+
+        self.spatial_stride = spatial_stride
+        self.temporal_stride = temporal_stride
+        self.dilation = dilation
+
+        self.style = style
+        self.all_frozen = all_frozen
+
+        self.stage_inflation = inflate
+        self.inflate_style = inflate_style
+        self.conv_cfg = conv_cfg
+        self.norm_cfg = norm_cfg
+        self.act_cfg = act_cfg
+        self.norm_eval = norm_eval
+        self.with_cp = with_cp
+        self.zero_init_residual = zero_init_residual
+
+        block, stage_blocks = self.arch_settings[depth]
+        stage_block = stage_blocks[stage]
+        planes = 64 * 2**stage
+        inplanes = 64 * 2**(stage - 1) * block.expansion
+
+        res_layer = self.make_res_layer(
+            block,
+            inplanes,
+            planes,
+            stage_block,
+            spatial_stride=spatial_stride,
+            temporal_stride=temporal_stride,
+            dilation=dilation,
+            style=self.style,
+            norm_cfg=self.norm_cfg,
+            conv_cfg=self.conv_cfg,
+            act_cfg=self.act_cfg,
+            inflate=self.stage_inflation,
+            inflate_style=self.inflate_style,
+            with_cp=with_cp,
+            **kwargs)
+
+        self.layer_name = f'layer{stage + 1}'
+        self.add_module(self.layer_name, res_layer)
+
+    def inflate_weights(self, logger):
+        self._inflate_weights(self, logger)
+
+    def _freeze_stages(self):
+        """Prevent all the parameters from being optimized before
+        ``self.frozen_stages``."""
+        if self.all_frozen:
+            layer = getattr(self, self.layer_name)
+            layer.eval()
+            for param in layer.parameters():
+                param.requires_grad = False
+
+    def init_weights(self, pretrained=None):
+        self._init_weights(self, pretrained)
+
+    def forward(self, x):
+        """Defines the computation performed at every call.
+
+        Args:
+            x (torch.Tensor): The input data.
+
+        Returns:
+            torch.Tensor: The feature of the input
+            samples extracted by the backbone.
+        """
+        res_layer = getattr(self, self.layer_name)
+        out = res_layer(x)
+        return out
+
+    def train(self, mode=True):
+        """Set the optimization status when training."""
+        super().train(mode)
+        self._freeze_stages()
+        if mode and self.norm_eval:
+            for m in self.modules():
+                if isinstance(m, _BatchNorm):
+                    m.eval()
+
+
+if 'mmdet' in dir():
+    MMDET_SHARED_HEADS.register_module()(ResNet3dLayer)
