@@ -1,3 +1,5 @@
+import warnings
+
 import torch
 import torch.nn as nn
 from mmcv.cnn import ConvModule, kaiming_init
@@ -8,6 +10,12 @@ from ...utils import get_root_logger
 from ..registry import BACKBONES
 from .resnet3d import ResNet3d
 
+try:
+    import mmdet  # noqa
+    from mmdet.models import BACKBONES as MMDET_BACKBONES
+except (ImportError, ModuleNotFoundError):
+    warnings.warn('Please install mmdet to use MMDET_BACKBONES')
+
 
 class ResNet3dPathway(ResNet3d):
     """A pathway of Slowfast based on ResNet3d.
@@ -24,7 +32,7 @@ class ResNet3dPathway(ResNet3d):
             Default: 8.
         fusion_kernel (int): The kernel size of lateral fusion.
             Default: 5.
-        **kwargs (keyword arguments): Keywork arguments for ResNet3d.
+        **kwargs (keyword arguments): Keywords arguments for ResNet3d.
     """
 
     def __init__(self,
@@ -122,9 +130,9 @@ class ResNet3dPathway(ResNet3d):
                 Default: 0.
             non_local_cfg (dict): Config for non-local module.
                 Default: ``dict()``.
-            conv_cfg (dict): Config for conv layers. Default: None.
-            norm_cfg (dict): Config for norm layers. Default: None.
-            act_cfg (dict): Config for activate layers. Default: None.
+            conv_cfg (dict | None): Config for conv layers. Default: None.
+            norm_cfg (dict | None): Config for norm layers. Default: None.
+            act_cfg (dict | None): Config for activate layers. Default: None.
             with_cp (bool): Use checkpoint or not. Using checkpoint will save
                 some memory while slowing down the training speed.
                 Default: False.
@@ -300,7 +308,7 @@ class ResNet3dPathway(ResNet3d):
             for param in m.parameters():
                 param.requires_grad = False
 
-            if (i != len(self.res_layers) and self.lateral):
+            if i != len(self.res_layers) and self.lateral:
                 # No fusion needed in the final stage
                 lateral_name = self.lateral_connections[i - 1]
                 conv_lateral = getattr(self, lateral_name)
@@ -308,9 +316,12 @@ class ResNet3dPathway(ResNet3d):
                 for param in conv_lateral.parameters():
                     param.requires_grad = False
 
-    def init_weights(self):
+    def init_weights(self, pretrained=None):
         """Initiate the parameters either from existing checkpoint or from
         scratch."""
+        if pretrained:
+            self.pretrained = pretrained
+
         # Override the init_weights of i3d
         super().init_weights()
         for module_name in self.lateral_connections:
@@ -343,8 +354,8 @@ def build_pathway(cfg, *args, **kwargs):
     pathway_type = cfg_.pop('type')
     if pathway_type not in pathway_cfg:
         raise KeyError(f'Unrecognized pathway type {pathway_type}')
-    else:
-        pathway_cls = pathway_cfg[pathway_type]
+
+    pathway_cls = pathway_cfg[pathway_type]
     pathway = pathway_cls(*args, **kwargs, **cfg_)
 
     return pathway
@@ -358,12 +369,13 @@ class ResNet3dSlowFast(nn.Module):
     <https://arxiv.org/abs/1812.03982>`_
 
     Args:
-        depth (int): Depth of resnet, from {18, 34, 50, 101, 152}.
         pretrained (str): The file path to a pretrained model.
         resample_rate (int): A large temporal stride ``resample_rate``
-            on input frames, corresponding to the :math:`\\tau` in the paper.
-            i.e., it processes only one out of ``resample_rate`` frames.
-            Default: 16.
+            on input frames. The actual resample rate is calculated by
+            multipling the ``interval`` in ``SampleFrames`` in the
+            pipeline with ``resample_rate``, equivalent to the :math:`\\tau`
+            in the paper, i.e. it processes only one out of
+            ``resample_rate * interval`` frames. Default: 8.
         speed_ratio (int): Speed ratio indicating the ratio between time
             dimension of the fast and slow pathway, corresponding to the
             :math:`\\alpha` in the paper. Default: 8.
@@ -431,9 +443,12 @@ class ResNet3dSlowFast(nn.Module):
         self.slow_path = build_pathway(slow_pathway)
         self.fast_path = build_pathway(fast_pathway)
 
-    def init_weights(self):
+    def init_weights(self, pretrained=None):
         """Initiate the parameters either from existing checkpoint or from
         scratch."""
+        if pretrained:
+            self.pretrained = pretrained
+
         if isinstance(self.pretrained, str):
             logger = get_root_logger()
             msg = f'load model from: {self.pretrained}'
@@ -454,8 +469,8 @@ class ResNet3dSlowFast(nn.Module):
             x (torch.Tensor): The input data.
 
         Returns:
-            tuple[torch.Tensor]: The feature of the input
-            samples extracted by the backbone.
+            tuple[torch.Tensor]: The feature of the input samples extracted
+                by the backbone.
         """
         x_slow = nn.functional.interpolate(
             x,
@@ -492,3 +507,7 @@ class ResNet3dSlowFast(nn.Module):
         out = (x_slow, x_fast)
 
         return out
+
+
+if 'mmdet' in dir():
+    MMDET_BACKBONES.register_module()(ResNet3dSlowFast)
