@@ -1,6 +1,14 @@
+import warnings
+
 from ..registry import BACKBONES
 from .resnet3d_slowfast import ResNet3dPathway
 
+try:
+    import mmdet  # noqa
+    from mmdet.models.builder import BACKBONES as MMDET_BACKBONES
+except (ImportError, ModuleNotFoundError):
+    warnings.warn('Please install mmdet to use MMDET_BACKBONES')
+
 
 @BACKBONES.register_module()
 class ResNet3dSlowOnly(ResNet3dPathway):
@@ -16,7 +24,7 @@ class ResNet3dSlowOnly(ResNet3dPathway):
             Default: 1.
         inflate (Sequence[int]): Inflate Dims of each block.
             Default: (0, 0, 1, 1).
-        **kwargs (keyword arguments): Keywork arguments for
+        **kwargs (keyword arguments): Keywords arguments for
             :class:`ResNet3dPathway`.
     """
 
@@ -40,3 +48,7 @@ class ResNet3dSlowOnly(ResNet3dPathway):
             **kwargs)
 
         assert not self.lateral
+
+
+if 'mmdet' in dir():
+    MMDET_BACKBONES.register_module()(ResNet3dSlowOnly)
