@@ -1,8 +1,21 @@
+import warnings
+
 import torch.nn as nn
-from mmcv.utils import build_from_cfg
+from mmcv.utils import Registry, build_from_cfg
 
 from .registry import BACKBONES, HEADS, LOCALIZERS, LOSSES, NECKS, RECOGNIZERS
 
+try:
+    from mmdet.models.builder import DETECTORS, build_detector
+except (ImportError, ModuleNotFoundError):
+    warnings.warn('Please install mmdet to use DETECTORS, build_detector')
+
+    # Define an empty registry and building func, so that can import
+    DETECTORS = Registry('detector')
+
+    def build_detector(cfg, train_cfg, test_cfg):
+        pass
+
 
 def build(cfg, registry, default_args=None):
     """Build a module.
@@ -23,8 +36,8 @@ def build(cfg, registry, default_args=None):
             build_from_cfg(cfg_, registry, default_args) for cfg_ in cfg
         ]
         return nn.Sequential(*modules)
-    else:
-        return build_from_cfg(cfg, registry, default_args)
+
+    return build_from_cfg(cfg, registry, default_args)
 
 
 def build_backbone(cfg):
@@ -59,8 +72,12 @@ def build_model(cfg, train_cfg=None, test_cfg=None):
     obj_type = args.pop('type')
     if obj_type in LOCALIZERS:
         return build_localizer(cfg)
-    elif obj_type in RECOGNIZERS:
+    if obj_type in RECOGNIZERS:
         return build_recognizer(cfg, train_cfg, test_cfg)
+    if obj_type in DETECTORS:
+        return build_detector(cfg, train_cfg, test_cfg)
+    raise ValueError(f'{obj_type} is not registered in '
+                     'LOCALIZERS, RECOGNIZERS or DETECTORS')
 
 
 def build_neck(cfg):
