@@ -20,7 +20,7 @@ class BMN(BaseLocalizer):
     Code Reference https://github.com/JJBOY/BMN-Boundary-Matching-Network
 
     Args:
-        temporal_dimension (int): Total frames selected for each video.
+        temporal_dim (int): Total frames selected for each video.
         boundary_ratio (float): Ratio for determining video boundaries.
         num_samples (int): Number of samples for each proposal.
         num_samples_per_bin (int): Number of bin samples for each sample.
@@ -350,10 +350,11 @@ class BMN(BaseLocalizer):
             label_end = label_end.to(device)
             return self.forward_train(raw_feature, label_confidence,
                                       label_start, label_end)
-        else:
-            return self.forward_test(raw_feature, video_meta)
 
-    def _get_interp1d_bin_mask(self, seg_tmin, seg_tmax, tscale, num_samples,
+        return self.forward_test(raw_feature, video_meta)
+
+    @staticmethod
+    def _get_interp1d_bin_mask(seg_tmin, seg_tmax, tscale, num_samples,
                                num_samples_per_bin):
         """Generate sample mask for a boundary-matching pair."""
         plen = float(seg_tmax - seg_tmin)
