@@ -1,8 +1,10 @@
 from .collect_env import collect_env
+from .gradcam_utils import GradCAM
 from .logger import get_root_logger
 from .misc import get_random_string, get_shm_dir, get_thread_id
+from .precise_bn import PreciseBNHook
 
 __all__ = [
     'get_root_logger', 'collect_env', 'get_random_string', 'get_thread_id',
-    'get_shm_dir'
+    'get_shm_dir', 'GradCAM', 'PreciseBNHook'
 ]
